unit UMain;

interface

{$I Delphi_Versions.inc}

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,FMX.Platform,FMX.Consts,
  inifiles,

  // spcific ECNativeMap
  FMX.uecNativeMapControl,
  FMX.uecNativeShape,FMX.uecgeoLocalise,FMX.uecMapUtil,FMX.uecNativePlaceLayer,
  FMX.uecNativePlace,
  //


  FMX.StdCtrls,System.UIConsts, FMX.Edit, FMX.Objects,

  // xe6
   System.Sensors,System.Sensors.Components,

  // for xe4 and xe5 use FMX.Sensors
  // FMX.Sensors,

  FMX.Gestures,System.IOUtils {$IFDEF XE7_OR_HIGHER}, FMX.Controls.Presentation{$ENDIF};

type
  TForm1 = class(TForm)
    destination: TEdit;
    address: TEdit;
    LocationSensor: TLocationSensor;
    LocationSwitch: TSwitch;
    Label1: TLabel;
    PanoramioSwitch: TSwitch;
    ln_pano: TLabel;
    pn_layers: TPanel;
    SatelliteSwitch: TSwitch;
    lb_satellite: TLabel;
    bar_options: TRectangle;
    pn_addresses: TPanel;
    lb_food: TLabel;
    FoodSwitch: TSwitch;
    bt_close_layers: TButton;
    bt_navigate: TButton;
    bt_close_navigate: TButton;
    btRoad: TImage;
    btLayer: TImage;
    btConfig: TImage;
    pn_config: TPanel;
    lb_cache: TLabel;
    CacheSwitch: TSwitch;
    Label2: TLabel;
    Label3: TLabel;
    btCloseConfig: TButton;
    pn_background: TRectangle;
    btClearCache: TButton;
    btClearMap: TButton;
    btSaveMap: TButton;
    Label4: TLabel;
    SwitchZoom: TSwitch;
    Label5: TLabel;
    SwitchInertia: TSwitch;
    procedure FormCreate(Sender: TObject);

    procedure BtFindClick(Sender: TObject);

    procedure btRoadClick(Sender: TObject);
    procedure LocationSwitchSwitch(Sender: TObject);
    procedure LocationSensorLocationChanged(Sender: TObject; const OldLocation,
      NewLocation: TLocationCoord2D);
    procedure FoodSwitchSwitch(Sender: TObject);
    procedure PanoramioSwitchSwitch(Sender: TObject);
    procedure SatelliteSwitchSwitch(Sender: TObject);

    procedure bt_close_layersClick(Sender: TObject);
    procedure bt_navigateClick(Sender: TObject);
    procedure btCloseConfigClick(Sender: TObject);
    procedure btConfigClick(Sender: TObject);
    procedure CacheSwitchSwitch(Sender: TObject);
    procedure btClearCacheClick(Sender: TObject);
   
    procedure bt_close_navigateClick(Sender: TObject);
    procedure pn_backgroundClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btClearMapClick(Sender: TObject);
    procedure btSaveMapClick(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure SwitchZoomSwitch(Sender: TObject);
    procedure SwitchInertiaSwitch(Sender: TObject);
   


  private


    FIniFile          : TStringList;

    FFileNameLog,
    FFileNameMap,
    FFileName         : string;


    FInfoWindowShapes : TECShapes;

    MyLocationShape   : TECShapePOI;

    FPlacesLayer : TECNativePlaceLayer;

    FMoveItem    : TECShape;



    procedure doOnDrawAllRoute(sender : TObject);

    procedure doShapeLocation(item:TECShape;const GeoResult:TECGeoResult;const Valid:boolean);

    function HandleAppEvent(AAppEvent: TApplicationEvent; AContext: TObject): Boolean;
  
    { Dclarations prives }
    procedure mapRoutePath(sender: TObject; const params: string;  routePath: TECRoutePath);

    procedure mapMapClick(sender: TObject; const Lat, Lng: Double);

    procedure mapShapeClick(sender: TObject; const item: TECShape);

    procedure mapBeforeUrl(sender: TObject; var Url: string);

    procedure mapLoad(sender: TObject; const GroupName: string;const FinishLoading: Boolean);

    procedure doEndAnimationMove(sender : TObject);


    procedure doOnPlaceClick(sender : TECShape);

    procedure OpenInfoWindow(const content:string;const Latitude,Longitude:single);

    procedure GetArcGisPlacesTile(var TileFilename:string;const x,y,z:integer);

    procedure mapPanoramioClick(sender: TObject; const Item: TECShape;
  const ownerId, ownerName, PhotoId, PhotoDate, PhotoTitle, PhotoUrl,
  copyright: string);

   function GetDirectoryCache : string;

   procedure Close_All_Panel;


   procedure SaveOptions;

   procedure UpdateLocalCache;
   procedure UpdateSatellite;
  public
    { Dclarations publiques }
  end;

var
  Form1 : TForm1;
  map   : TECNativeMap;

implementation

{$R *.fmx}


function GetRandomColor:TAlphaColor;
begin

 TAlphaColorRec(result).R := random(255);
 TAlphaColorRec(result).G := random(255);
 TAlphaColorRec(result).B := random(255);
 TAlphaColorRec(result).A := 255;

end;

procedure TForm1.Close_All_Panel;
begin

  SaveOptions;

  pn_layers.visible    := false;
  pn_addresses.visible := false;
  pn_config.visible    := false;

  pn_background.Visible:= false;



end;


// ==================== Create / Initialize ====================================

procedure TForm1.FormCreate(Sender: TObject);
var
  ScreenSvc : IFMXScreenService;
  LSize     : TPointF;

  ColTiles,
  RowTiles :integer;

  KmlFiles : TStringDynArray;

  var aFMXApplicationEventService: IFMXApplicationEventService;
begin


 if TPlatformServices.Current.SupportsPlatformService(IFMXApplicationEventService, IInterface(aFMXApplicationEventService)) then
    aFMXApplicationEventService.SetApplicationEventHandler(HandleAppEvent)  ;


 FIniFile    := TStringList.Create;
 FFilename   := TPath.Combine(TPath.GetSharedDocumentsPath, 'config.ini');
 FFileNameMap:= TPath.Combine(TPath.GetSharedDocumentsPath, 'map.txt');
 FFilenameLog:= TPath.Combine(TPath.GetSharedDocumentsPath, 'log.txt');




 // dynamic create map
 map := TECNativeMap.Create(self);
 map.Parent := self;
 map.Align  := TalignLayout.alClient;
 map.visible:= true;


 //
 pn_background.Parent := map;



 //map.OnLoad         := mapLoad;
 map.OnRoutePath    := mapRoutePath;
 map.OnMapLongClick := mapMapClick;

 map.OnShapeClick   := mapShapeClick;

 map.OnBeforeUrl    := mapBeforeUrl;



 // group infowindows
 FInfoWindowShapes := map.Group['mywindows'];
 FInfoWindowShapes.Zindex := 2;


 // default infowindow
 FInfoWindowShapes.InfoWindows.add(0,0,'');
 FInfoWindowShapes.InfoWindows[0].zindex := 100;
 FInfoWindowShapes.InfoWindows[0].Width  := 200;
 FInfoWindowShapes.InfoWindows[0].Visible := false;

 MyLocationShape   := nil;

 // search Layer

 FPlacesLayer               := TECNativePlaceLayer.create(map,'PLACES_LAYER');
 FPlacesLayer.Shapes.zindex := 2;
 FPlacesLayer.OnPLaceClick  := doOnPlaceClick;

 FPlacesLayer.Visible := false;
 // find restaurant
 FPlacesLayer.Search  := 'node[amenity=restaurant]';
 FPlacesLayer.MarkerFilename := 'http://google-maps-icons.googlecode.com/files/restaurant.png';


 map.OnPanoramioClick := mapPanoramioClick;

 // response for  location shape
 map.shapes.OnShapeLocation := doShapeLocation;


 // adapt TileCacheSize to screen Size

 if TPlatformServices.Current.SupportsPlatformService(IFMXScreenService, IInterface(ScreenSvc)) then
 begin
  LSize := ScreenSvc.GetScreenSize;

  ColTiles := round(LSize.X / 256) + 1;
  RowTiles := round(LSize.Y / 256) + 1;



  Map.TileCacheSize := 128;//8 * ColTiles * RowTiles;


 end;




  // load data map

  map.LoadFromFile(FFileNameMap);

  // load parameters

   if fileexists(FFilename) then
   FIniFile.LoadFromFile(FFilename);


  CacheSwitch.IsChecked     := StrToBoolDef(FIniFile.Values['localcache'],true);
  FoodSwitch.IsChecked      := StrToBoolDef(FIniFile.Values['food'],false);
  SatelliteSwitch.IsChecked := StrToBoolDef(FIniFile.Values['satellite'],false);
  PanoramioSwitch.IsChecked := StrToBoolDef(FIniFile.Values['panoramio'],false);
  SwitchZoom.IsChecked      := StrToBoolDef(FIniFile.Values['zoomeffect'],true);
  SwitchInertia.IsChecked   := StrToBoolDef(FIniFile.Values['inertiaeffect'],false);

  map.InertiaScroll := SwitchInertia.ischecked;
  map.ZoomEffect    := SwitchZoom.IsChecked;
  UpdateLocalCache;
  UpdateSatellite;
  map.PanoramioLayer := PanoramioSwitch.IsChecked;




  // load first kml if exist
 (*
 KmlFiles := System.IOUtils.TDirectory.GetFiles(TPath.GetSharedDocumentsPath,'*.kml');

 if Length(KmlFiles)>0 then
 begin
   map.LoadFromFile(KmlFiles[0]);
 end;
 *)



  Close_All_Panel;



end;

procedure TForm1.mapLoad(sender: TObject; const GroupName: string;
  const FinishLoading: Boolean);
begin
 if assigned(sender) and (sender is TECShapes) then
  TECShapes(sender).fitBounds;
end;

function TForm1.HandleAppEvent(AAppEvent: TApplicationEvent; AContext: TObject): Boolean;
begin

  {case AAppEvent of

   //TApplicationEvent.LowMemory: showmessage('low memory');//map.Log.add('low memory');

   (* aeFinishedLaunching: ;
    aeBecameActive:;
    aeWillBecomeInactive: ;
    aeEnteredBackground: ;
    aeWillBecomeForeground: ;
    aeWillTerminate: ;
    aeLowMemory: ;//map.Log.add('low memory');
    aeTimeChange: ;
    aeOpenURL: ; *)
  end;
  Result := True; }
end;


// directory cache

function TForm1.GetDirectoryCache : string;
begin
  result := TPath.Combine(TPath.GetSharedDocumentsPath, 'cache');
end;



procedure TForm1.SaveOptions;
begin

  FIniFile.Values['localcache']    := BoolToStr(CacheSwitch.IsChecked);
  FIniFile.Values['food']          := BoolToStr(FoodSwitch.IsChecked) ;
  FIniFile.Values['satellite']     := BoolToStr(SatelliteSwitch.IsChecked) ;
  FIniFile.Values['panoramio']     := BoolToStr(PanoramioSwitch.IsChecked) ;
  FIniFile.Values['zoomeffect']    := BoolToStr(SwitchZoom.IsChecked);
  FIniFile.Values['inertiaeffect'] := BoolToStr(SwitchInertia.IsChecked);

  FIniFile.SaveToFile(FFilename);

end;



procedure TForm1.SwitchInertiaSwitch(Sender: TObject);
begin
 map.InertiaScroll := SwitchInertia.ischecked;
end;

procedure TForm1.SwitchZoomSwitch(Sender: TObject);
begin
 map.ZoomEffect := SwitchZoom.IsChecked;
end;

procedure TForm1.UpdateLocalCache;
begin

 if CacheSwitch.IsChecked then
  Map.LocalCache := GetDirectoryCache
 else
  Map.LocalCache := '';

end;




procedure TForm1.FormDeactivate(Sender: TObject);
begin
//map.SaveToFile(FFileNameMap);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 FIniFile.Free;
end;








// -----------------------------------------------------------------------------



//  ==================  LocalCache =============================================

// Switch Cache
procedure TForm1.CacheSwitchSwitch(Sender: TObject);
begin

  UpdateLocalCache;


  btClearCache.visible  := CacheSwitch.IsChecked;


  if pn_config.visible then
   pn_config.SetFocus;
end;






procedure TForm1.btClearCacheClick(Sender: TObject);
begin
  System.IOUtils.TDirectory.Delete(GetDirectoryCache,true);
end;

procedure TForm1.btClearMapClick(Sender: TObject);
begin
 Map.Shapes.Clear;
 map.Group['mywindows'].clear;
 FPlacesLayer.Shapes.Clear;
end;

// -----------------------------------------------------------------------------


// ============ actions Panels      ============================================
procedure TForm1.btCloseConfigClick(Sender: TObject);
begin
 //map.Log.savetofile(FFilenameLog);
 Close_All_Panel;
end;

procedure TForm1.btConfigClick(Sender: TObject);
begin
 Close_All_Panel;
 pn_background.Visible:= true;
 pn_config.Visible    := true;
end;

procedure TForm1.BtFindClick(Sender: TObject);
begin
 Close_All_Panel;
 pn_background.Visible:= true;
 pn_layers.visible := true;
end;



procedure TForm1.btRoadClick(Sender: TObject);
begin
 Close_All_Panel;
 pn_background.Visible:= true;
 pn_addresses.visible := true;

 Address.SetFocus;

end;


procedure TForm1.btSaveMapClick(Sender: TObject);
begin
  map.SaveToFile(FFileNameMap);
end;


procedure TForm1.bt_close_layersClick(Sender: TObject);
begin
 Close_All_Panel;
end;

procedure TForm1.bt_close_navigateClick(Sender: TObject);
begin
 Close_All_Panel;
end;









// ============  Satellite Layer ===============================================
procedure TForm1.SatelliteSwitchSwitch(Sender: TObject);
begin

 UpdateSatellite;


 if pn_layers.visible then
  pn_layers.SetFocus;

end;


procedure TForm1.UpdateSatellite;
begin
  if SatelliteSwitch.IsChecked then
 begin
  map.tileServer        := tsBingAerialLabels;//tsArcGisWorldImagery;
  // add overlay tiles for places
 // map.AddOverlayTiles(GetArcGisPlacesTile,'World_Places_Tile');
 end

 else

 begin

   map.RemoveOverlayTiles;
   map.TileServer := tsOSM;

 end;

end;

// get satellite tiles
procedure TForm1.GetArcGisPlacesTile(var TileFilename:string;const x,y,z:integer);
begin
 TileFilename := format('http://services.arcgisonline.com/ArcGIS/rest/services/Reference/World_Boundaries_and_Places/MapServer/tile/%d/%d/%d.png',[z,y,x]);
end;

// -----------------------------------------------------------------------------



// ============== Panoramio Layer ==============================================
procedure TForm1.PanoramioSwitchSwitch(Sender: TObject);
begin
 map.PanoramioLayer := PanoramioSwitch.IsChecked;

 if pn_layers.visible then
  pn_layers.SetFocus;
end;


//  Event Click on panoramio photo
procedure TForm1.mapPanoramioClick(sender: TObject; const Item: TECShape;
  const ownerId, ownerName, PhotoId, PhotoDate, PhotoTitle, PhotoUrl,
  copyright: string);
begin

 OpenInfoWindow( '<img src="'+PanoramioPhotoUrlSize(photourl,pimSmall)+'" >'+
                 '<br><h2>'+PhotoTitle+'</h2>',item.Latitude,item.Longitude);

end;

// -----------------------------------------------------------------------------



procedure TForm1.pn_backgroundClick(Sender: TObject);
begin
 Close_All_Panel;
end;

// -----------------------------------------------------------------------------



// ================ my location ================================================

procedure TForm1.LocationSensorLocationChanged(Sender: TObject;
  const OldLocation, NewLocation: TLocationCoord2D);
begin



  LocationSensor.Active    := false;
  LocationSwitch.IsChecked := false;



  MyLocationShape.SetPosition(NewLocation.Latitude,NewLocation.longitude);

  MyLocationShape.Visible := true;


  if not MyLocationShape.ShowOnMap then
   Map.setCenter(MyLocationShape.Latitude,MyLocationShape.Longitude);

  // 200 meter around you
  map.fitBoundsRadius(NewLocation.Latitude,NewLocation.longitude,0.2);



end;

procedure TForm1.LocationSwitchSwitch(Sender: TObject);
 var i:integer;
begin

 // create POI if not
 if MyLocationShape = nil then
 begin
   i :=   map.shapes.Pois.add( 0,0);
   MyLocationShape          := map.shapes.Pois[i];
   MyLocationShape.POIShape := poiStar;
   MyLocationShape.width    := 48;
   MyLocationShape.height   := 48;

   MyLocationShape.Visible  := false;


  // illimited flash,  TECAnimationShapeColor is is automatically released
   MyLocationShape.animation := TECAnimationShapeColor.create(0);
 end;

 LocationSensor.Active  := LocationSwitch.IsChecked;



end;

// -----------------------------------------------------------------------------


// ============== global InfoWindows ===========================================
procedure TForm1.OpenInfoWindow(const content:string;const Latitude,Longitude:single);
begin


 if FInfoWindowShapes.InfoWindows.count = 0 then
 begin
 FInfoWindowShapes.InfoWindows.add(Latitude,Longitude,content);
 FInfoWindowShapes.InfoWindows[0].zindex := 100;
 end ;



  FInfoWindowShapes.BeginUpdate;

   FInfoWindowShapes.InfoWindows[0].content := Content;
   FInfoWindowShapes.InfoWindows[0].SetPosition(Latitude,Longitude);


 FInfoWindowShapes.zindex := map.TopGroupZIndex;;

 FInfoWindowShapes.InfoWindows[0].closebutton := true;

 FInfoWindowShapes.InfoWindows[0].Visible := true;

 FInfoWindowShapes.EndUpdate;

 FInfoWindowShapes.InfoWindows[0].CenterOnMap;


end;

// -----------------------------------------------------------------------------







// ============ Restaurant Layer ===============================================

procedure TForm1.FoodSwitchSwitch(Sender: TObject);
begin
 FPlacesLayer.Visible := FoodSwitch.IsChecked;

 if pn_layers.visible then
 pn_layers.SetFocus;
end;


//  event click on place shape ( restaurant )
procedure TForm1.doOnPlaceClick(sender : TECShape);
var
    pResult : TECPlaceResult;
    r,
    n,
    Content : string;
    i       : integer;
begin

   // here Sender and Sender.Item are allway assigned, and Sender.Item is TECPlaceResult
   pResult :=TECPlaceResult(Sender.item) ;


   content := '<img src="'+UrlStaticGoogleStreetView(Sender.Latitude,Sender.Longitude,300,200)+'"><br>';

  (* for i:=0 to pResult.CountResult-1 do
   begin



    n := pResult.NameResult[i];
    r := pResult.Result[n];
    if length(n)<9 then
     n := n+'<tab="65">';

    content := content+'<b>'+n+'</b>: '+r+'<br>';

   end; *)

   n := pResult.Result['name'] ;
   if n = '' then
   n := pResult.Result['types'] ;

   if n<>'' then
   Content := Content+'<h2>'+n+'</h2>';



  OpenInfoWindow(Content,Sender.Latitude,Sender.Longitude);



end;
// -----------------------------------------------------------------------------





// ============ Long Click on Map for Add Marker ===============================

procedure TForm1.mapMapClick(sender: TObject; const Lat, Lng: Double);
var i,j:integer;

  M    : TECShapeMarker;
  P    : TECShapePOI;
  anim : TECAnimationFadePoi ;
begin



 j := random(3);


 if j=1 then
 begin
  // POI marker

  i := map.shapes.pois.add(lat,lng);

  P := map.shapes.pois[i];

  P.width := 48;
  P.height:= 48;


  //P.hint := 'Poi n'+inttostr(i);

  P.Draggable := true;

  P.Color := GetRandomColor;

       case random(4) of
        0 : P.POIShape := poiEllipse;
        1 : P.POIShape := poiStar;
        2 : P.POIShape := poiRect;
        3 : P.POIShape := poiTriangle;
       // 4 : map.shapes.pois[i].POIShape := poiOwnerDraw;
       end;


  // anim
  if random(2)=0 then
  begin

   anim := TECAnimationFadePoi.Create;
   anim.MaxSize   := 48;
   anim.StartSize := 8 ;

   anim.StartOpacity := 95;

   P.Animation := anim;

  end;

 end

  else

  begin

    i :=   map.shapes.markers.add( Lat,Lng);

    M :=  map.shapes.markers[i];


    M.Draggable := true;

    if j=0 then
    begin

      // standard marker

      M.Color := GetRandomColor;
      M.HoverColor := GetHighlightColorBy(M.Color,32);

      // 20 flashing ,  TECAnimationShapeColor is is automatically released
      M.animation := TECAnimationShapeColor.create(20);
    end

    else

    begin
      // add image to marker

      case random(7) of
         0 : M.Filename := 'http://maps.google.com/mapfiles/ms/icons/yellow-dot.png';
         1 : M.Filename := 'http://maps.google.com/mapfiles/ms/icons/orange-dot.png';
         2 : M.Filename := 'http://maps.google.com/mapfiles/ms/icons/red-dot.png';
         3 : M.Filename := 'http://maps.google.com/mapfiles/ms/icons/pink-dot.png';
         4 : M.Filename := 'http://maps.google.com/mapfiles/ms/icons/purple-dot.png';
         5 : M.Filename := 'http://maps.google.com/mapfiles/ms/icons/green-dot.png';
         6 : M.Filename := 'http://maps.google.com/mapfiles/ms/icons/blue-dot.png';
         7 : M.Filename := 'http://maps.google.com/mapfiles/ms/icons/ltblue-dot.png';
       end;
    end;



 end;


end;
// -----------------------------------------------------------------------------



// =============== Click On Marker =============================================

procedure TForm1.mapShapeClick(sender: TObject; const item: TECShape);
begin

  if not assigned(item) then exit;

 // Asynchronous search address,
 //  when it is found map.shapes.OnShapeLocation is triggered

  item.Location;

end;


// triggered when item.Location done
procedure TForm1.doShapeLocation(item:TECShape;const GeoResult:TECGeoResult;const Valid:boolean);
var
    s:string;
    lat,lng:double;
begin

  if assigned(item) then
  begin

     lat := item.Latitude;
     lng := item.Longitude;

     s := '<img src="'+UrlStaticGoogleStreetView(lat,lng,300,200)+'">'
           +'<br>'+ GeoResult.Address;

   if Map.Zoom<18 then
   begin
    s := s+'<br><br><a href="#'+inttostr(18)+'/'+doubleToStrDigit(item.Latitude,6)+'/'+DoubleToStrDigit(item.Longitude,6)+'"><font hcolor=FF0000 >ZoomToStreet</font></a>';
   end;

    OpenInfoWindow(s,Lat,Lng);

  end else exit;

end;

// call by clickink on link "Zoom to street" in InfoWindow
 // close InfoWindow if exists
procedure TForm1.mapBeforeUrl(sender: TObject; var Url: string);
begin
  if (FInfoWindowShapes.InfoWindows.count > 0) then
  FInfoWindowShapes.InfoWindows[0].Visible := false;
end;
// -----------------------------------------------------------------------------





// =================== Navigate ================================================

procedure TForm1.bt_navigateClick(Sender: TObject);
begin

 Close_All_Panel;



 if( Address.Text<>'' ) and (Destination.Text<>'') then
  // Asynchronous add route, when it is found map.OnRoutePath is triggered
   map.GetAsyncRoutePathByAdress(Address.Text,Destination.Text)
 else
 if( Address.Text<>'' ) then
  map.address := Address.Text
 else
  ShowMessage('Enter addresses');

end;


// triggered when  map.GetAsyncRoutePathByAdress is ok
procedure TForm1.mapRoutePath(sender: TObject; const params: string;
  routePath: TECRoutePath);
var line   : TECShapeLine;
    DrawingRoute : TECAnimationDrawPath;
begin

 // test is valid route
 if routePath.Distance=0 then
 begin
  ShowMessage('Route not found !');
  exit;
 end;



 // add line on default group ''
 line := map.addRoute(routePath,'');
 line.weight  := 4;


 //line.opacity := 50;
 line.color   := GetRandomColor;

 // add border
line.BorderSize  := 2;
line.BorderColor := GetShadowColorBy(line.color,32) ;
// line.Editable := true;


 // see the totality of the road

 line.fitBounds;

 // draw route in 3s
 // DrawingRoute will be automatically deleted
 DrawingRoute := TECAnimationDrawPath.Create(3000);
 DrawingRoute.OnEndAnimation := doOnDrawAllRoute;

 line.Animation := DrawingRoute;

 line.hint := ' Distance <tab="100">: '+DoubleToStrDigit(routepath.Distance/1000,2)+' km' + '<br>'+
              'Time <tab="100">: '+DoubleToStrDigit(routePath.Duration/3600,2)+'h';



end;

// call by TECAnimationDrawPath  when route is 100% draw
procedure TForm1.doOnDrawAllRoute(sender : TObject);
var line   : TECShapeLine;
    moving : TECAnimationMoveOnPath;
    i      : integer;
    iw     : TECShapeInfoWindow;
begin

 if not assigned(sender) or not(Sender is TECShapeLine) then exit;

 line := sender as TECShapeLine;

// show hotspot text information
 line.ShowText := true;



 // change default poi start and finish of route

if line.shapes.Pois.count>1 then
 begin
   i := line.shapes.markers.add(line.shapes.pois[0].latitude,line.shapes.pois[0].longitude);
   line.shapes.markers[i].filename := 'http://www.helpandweb.com/flag_blue.png';
   line.shapes.markers[i].hint := line.shapes.pois[0].hint;

   i := line.shapes.markers.add(line.shapes.pois[1].latitude,line.shapes.pois[1].longitude);
   line.shapes.markers[i].XAnchor  := 0;
   line.shapes.markers[i].filename := 'http://www.helpandweb.com/checkered_flag.png';

   line.shapes.markers[i].hint := line.shapes.pois[1].hint;

   // hide default start and finish
   line.shapes.pois[0].visible := false;
   line.shapes.pois[1].visible := false;
 end;



 // add triangle moving on route

 // create triangle
  i := map.shapes.pois.Add(0,0);
  map.shapes.pois[i].POIShape := poiTriangle;
  map.shapes.pois[i].width := 18;
  map.shapes.pois[i].height:= 24;
  map.shapes.pois[i].color := GetShadowColorBy(line.color,32);




 // animation move on line
 // start km 0
 //speed at 50 + random(5)*10  km/h
 moving := TECAnimationMoveOnPath.create(line,0,50 + random(5)*10);

 //  directing the tip of the triangle in the direction of travel
 moving.heading := true;


 // The release of the animation is automatically

 map.shapes.pois[i].animation := moving;




 // create InfoWindow and keep is index
 map.shapes.pois[i].tag := FInfoWindowShapes.InfoWindows.add(10,10,'');

 FMoveItem := map.shapes.pois[i];

 iw := FInfoWindowShapes.InfoWindows[map.shapes.pois[i].tag];

 iw.CloseButton := false;
 iw.ContentCenter   := true;

 iw.color       := claBeige;
 iw.Style       := iwsRectangle;
 // infowindow on top of all shapes
 iw.Zindex        := map.TopGroupZIndex;

 // infowindow show distance / duration

 i := line.Count div 2;

 iw.SetPosition(line.Path[i].Latitude,line.Path[i].Longitude);

  // InfoWindow Content can interpret basic html commands

  iw.Content   := line.hint;

  iw.Visible := true;


  // The release of the animation is automatically
  iw.Animation := TECAnimationAutoHide.create;

  // auto hide infowindow after 5s
  TECAnimationAutoHide(iw.Animation).MaxTiming := 5000;


 // fired when arrived at end (or at start)
 moving.OnDriveUp := doEndAnimationMove;
 // run animation
 moving.start;

end;




// fired when mobile arrived
procedure TForm1.doEndAnimationMove(sender : TObject);
var iw : TECShapeInfoWindow;
begin

  // reversing the direction of movement
  TECAnimationMoveOnPath(TECShape(sender).animation).ComeBack := not TECAnimationMoveOnPath(TECShape(sender).animation).ComeBack;
  // re run mobile
  TECAnimationMoveOnPath(TECShape(sender).animation).Stop := false;



  // show infowindow

  iw := FInfoWindowShapes.InfoWindows[TECShape(sender).Tag];

  iw.SetPosition(TECShape(sender).Latitude,TECShape(sender).Longitude);

  // InfoWindow Content can interpret basic html commands
  iw.Content   := '<img src="http://www.helpandweb.com/icon-damier.png" width=42 height=21> <b>Arrived !</b>'  ;

  iw.Visible := true;


  // The release of the animation is automatically
  iw.Animation := TECAnimationAutoHide.create;

  // auto hide infowindow after 2s
  TECAnimationAutoHide(iw.Animation).MaxTiming := 2000;
end;


end.
